#include <avr/io.h>
#include <avr/boot.h>
#ifdef MODE_LCD
	#include "lcd.h"
#endif
#include "mmc.h"
#include "delay.h"
#include "types.h"


#define PAGESIZEB 128

#ifdef __AVR_ATmega168__
	#define ROMSIZEB 16384
	#define BOOTSIZEB 2048
#endif

#ifdef __AVR_ATmega88__
	#define ROMSIZEB 8192
	#define BOOTSIZEB 2048
#endif

#ifndef F_CPU
	#define F_CPU 8000000
#endif


unsigned char buf[PAGESIZEB];


void main(void) {
	u16 sector = TCNT1, blkcnt = OCR1A<<2, cnt=0, i;
	
	cli();
	asm volatile ( "out 0x2a, __zero_reg__" );
	asm volatile ( "out 0x2b, __zero_reg__" );

#ifdef MODE_LCD
	lcd_clear();
	delay_ms(5);
	lcd_string(DISP_MAIN, 0);
	delay_ms(100);
#endif
	if (sector == 0 || blkcnt == 0)
		while(1);
#ifdef MODE_LCD
	lcd_string(DISP_WRITE, 1);
#endif
	mmc_load_start(sector);
	while (blkcnt) {
#ifdef MODE_LCD
		lcd_setcur(14, 1);
		lcd_hex_u16(cnt);
#endif
		mmc_read_block(buf, PAGESIZEB);
		u32 page = cnt<<7;
		boot_page_erase(page);
		boot_spm_busy_wait();

		u08 wbufaddr = 0;
		for (i=0; i<SPM_PAGESIZE; i+=2)
		{
		    // Set up little-endian word.
		    u16 w = buf[wbufaddr++];
		    w += buf[wbufaddr++] << 8;
		    boot_page_fill(page + i, w);
		}
		boot_page_write(page);     // Store buffer in flash page.
		boot_spm_busy_wait();       // Wait until the memory is written.

		cnt++;
		blkcnt--;
	}
	mmc_complete_read();
#ifdef MODE_LCD
	lcd_string(DISP_DONE, 1);
	while(1);
#endif
}

